;
; Velleman K8048 PIC16F648A PS/2 to RS-232 Demo
;
; Copyright (c) 2005-2013 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE for details.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Pinout
; ------
; RA2 AN2   VREF 1----18 RA1 AN1
; RA3 AN3   CMP1 2    17 RA0 AN0
; RA4 T0CKI CMP2 3    16 RA7 OSC1  CLKIN
; RA5 !MCLR VPP  4    15 RA6 OSC2  CLKOUT
; VSS GND        5    14 VDD VCC
; RB0 INT        6    13 RB7 T1OSI PGD
; RB1 RX    DT   7    12 RB6 T1OSO T1CKI PGC
; RB2 TX    CK   8    11 RB5
; RB3 CCP1       9----10 RB4 PGM
;
; K8048 18P      ALT 14P
; ----- ---      --- ---
; LD1   RB0 (6)
; LD2   RB1 (7)  RX  9
; LD3   RB2 (8)  TX  8
; LD4   RB3 (9)
; LD5   RB4 (10)
; LD6   RB5 (11)
; PGC   RB6 (12)
; PGD   RB7 (13)
; SW1   RA0 (17)
; SW2   RA1 (18)
; SW3   RA2 (1)
; SW4   RA3 (2)
; VPP   RA5 (4)
; OSC2  RA6 (15)
; OSC1  RA7 (16)
; GND   VSS (5)
; +5V   VDD (14)
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
                LIST    P=PIC16F648A
ERRORLEVEL      -302
#INCLUDE        "p16f648a.inc"
#INCLUDE        "device.inc"                ;DEVICE CONFIG
#INCLUDE        "const.inc"                 ;CONSTANTS
#INCLUDE        "macro.inc"                 ;MACROS
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; K8048 PIC16F648A PS/2 to RS-232 Demo
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Config
;
  __CONFIG _CP_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTOSC_OSC_NOCLKOUT
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Constants
;
; HS XTAL = 11.0592 MHz or HS XTAL = 7.3728 MHz
;   CONSTANT CLOCK = 7372800
;   CONSTANT CLOCK = 11059200
;
; XT XTAL = 4000000Hz
;   CONSTANT CLOCK = 4000000
;
;*ASYNC BAUD RATE (BRGH = 0, BRG16 = 0) SPBRGH:SPBRG = FOSC / BAUD RATE / 64 - 1
; ASYNC BAUD RATE (BRGH = 1, BRG16 = 0) SPBRGH:SPBRG = FOSC / BAUD RATE / 16 - 1
; ASYNC BAUD RATE (BRGH = 0, BRG16 = 1) SPBRGH:SPBRG = FOSC / BAUD RATE / 16 - 1
; ASYNC BAUD RATE (BRGH = 1, BRG16 = 1) SPBRGH:SPBRG = FOSC / BAUD RATE / 4  - 1
;
; 75, 150, 300, 600, 1200, 1800, 2400, 3600, 4800, 7200, 9600, 14400, 19200,
; 28800, 38400, 57600, 115200, 230400, 460800, 614400, 921600, 1228800,
; 2457600, 3000000, 6000000 (pl2303 Linux)
;
;   CONSTANT SPEED = CLOCK / 230400 / 16 - 1
;   CONSTANT SPEED = CLOCK / 9600   / 64 - 1
;
; INTOSC = 4MHz
    CONSTANT CLOCK = 4000000
; BAUD RATE
    CONSTANT SPEED = CLOCK / 1200 / 64 - 1
    CONSTANT SAMPLERATE = 20 ;20 SAMPLES/SECOND @1200 BAUD
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Variables
;
CBLOCK          0x20                        ;RAM 0x20..0x6F
    BYTE1
    BYTE2
    BYTE3
ENDC
#INCLUDE        "shadow.inc"
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Begin
;
                ORG     0x0000
                GOTO    INIT
                ORG     0x0004
                RETFIE
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; PS/2 MOUSE TEST
;
#INCLUDE        "delay.inc"                 ;DELAY COUNTERS
#INCLUDE        "ps2.inc"                   ;PS/2 PROTOCOL
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Initialise
;
INIT            BANKSEL BANK0

                CLRF    LATA                ;INIT PORTA SHADOW
                CLRF    PORTA               ;INIT PORTA

                CLRF    LATB                ;INIT PORTB SHADOW
                CLRF    PORTB               ;INIT PORTB

                CLRF    INTCON              ;DISABLE INTERRUPTS
                CLRF    PIR1                ;CLEAR INTERRRUPT FLAGS

                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON

                BANKSEL BANK1

                CLRF    PIE1                ;DISABLE PERIPHERAL INTERRUPTS

                MOVLW   B'10001111'         ;DISABLE PULLUPS/WATCHDOG PRESCALE
                MOVWF   OPTION_REG

                MOVLW   B'11111111'         ;PORT A I/P
                MOVWF   TRISA               ;INIT TRISA

                MOVLW   B'11111111'         ;PORT B I/P TX/RX UART
                MOVWF   TRISB               ;INIT TRISB
;UART
                MOVLW   LOW SPEED           ;BAUD RATE GENERATOR SPEED
                MOVWF   SPBRG

                MOVLW   1<<TXEN             ;TX ENABLE
                MOVWF   TXSTA

                BANKSEL BANK0

                MOVLW   1<<SPEN | 1<<CREN   ;PORT ENABLE + CONTINUOUS RX
                MOVWF   RCSTA

                MOVF    RCREG,W             ;DISCARD
                MOVF    RCREG,W             ;DISCARD
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Serial mouse ident
; 
                MOVLW   'M'
                CALL    TRANSMIT
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; PS/2 Init
; 
PS2INIT         MOVLW   PS2_CMD_RESET_DIS
                CALL    PS2SEND             ;FA
                BTFGS   STATUS,C,PS2INIT

                MOVLW   PS2_CMD_RESET_BAT
                CALL    PS2SEND             ;FA
                BTFGS   STATUS,C,PS2INIT

                CALL    PS2RECEIVE          ;AA
                BTFGS   STATUS,C,PS2INIT

                CALL    PS2RECEIVE          ;00
                BTFGS   STATUS,C,PS2INIT

                MOVLW   PS2_CMD_SETRATE
                CALL    PS2SEND             ;FA
                BTFGS   STATUS,C,PS2INIT

                MOVLW   SAMPLERATE
                CALL    PS2SEND             ;FA
                BTFGS   STATUS,C,PS2INIT

                MOVLW   PS2_CMD_ENABLE
                CALL    PS2SEND             ;FA
                BTFGS   STATUS,C,PS2INIT
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Main loop
;
MAINLOOP        CALL    PS2RECEIVE          ;GET PS/2 PACKET
                BTFGS   STATUS,C,MAINLOOP
                MOVF    BUFFER,W
                MOVWF   BYTE1

                CALL    PS2RECEIVE          ;X
                BTFGS   STATUS,C,PS2INIT
                MOVF    BUFFER,W
                MOVWF   BYTE2

                CALL    PS2RECEIVE          ;Y
                BTFGS   STATUS,C,PS2INIT
                MOVF    BUFFER,W
                SUBLW   0                   ;2s COMPLEMENT
                MOVWF   BYTE3

; CONVERT TO SERIAL MOUSE PACKET
                MOVLW   B'11000000'
                BTFSC   BYTE1,0             ;LEFT BUTTON
                IORLW   B'00100000'
                BTFSC   BYTE1,1             ;RIGHT BUTTON
                IORLW   B'00010000'

                BTFSC   BYTE2,6             ;X6
                IORLW   B'00000001'
                BTFSC   BYTE2,7             ;X7
                IORLW   B'00000010'
                BTFSC   BYTE3,6             ;Y6
                IORLW   B'00000100'
                BTFSC   BYTE3,7             ;Y7
                IORLW   B'00001000'
                CALL    TRANSMIT            ;SEND

                MOVF    BYTE2,W             ;X0..X5
                ANDLW   B'00111111'
                IORLW   B'10000000'
                CALL    TRANSMIT            ;SEND

                MOVF    BYTE3,W             ;Y0..Y5
                ANDLW   B'00111111'
                IORLW   B'10000000'
                CALL    TRANSMIT            ;SEND

                GOTO    MAINLOOP            ;REPEAT...

TRANSMIT        BTFSS   PIR1,TXIF           ;WAIT FOR TX
                GOTO    TRANSMIT
                MOVWF   TXREG               ;SEND DATA
                RETURN
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Interrupt service routine
;
#IF 0
CBLOCK          0x7D                        ;RAM 0x7D..0x7F
    W_SAVE
    S_SAVE
    F_SAVE
ENDC
ISR             MOVWF   W_SAVE              ;SAVE WORKING REGISTER
                MOVF    STATUS,W
                MOVWF   S_SAVE              ;SAVE STATUS
                MOVF    FSR,W
                MOVWF   F_SAVE              ;SAVE FSR

                NOP

                MOVF    F_SAVE,W            ;RESTORE FSR
                MOVWF   FSR
                MOVF    S_SAVE,W            ;RESTORE STATUS
                MOVWF   STATUS
                MOVF    W_SAVE,W            ;RESTORE WORKING REGISTER
                RETFIE
#ENDIF
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                END
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; vim: shiftwidth=4 tabstop=4 softtabstop=4 expandtab
;
