/*
 * Velleman K8048 Programmer for FreeBSD and others.
 *
 * Copyright (c) 2005-2013 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE for details.
 */

#include "k8048.h"

#if 0
#define DEBUG
#endif

/*
 * Configuration
 */
void
getconf(struct k8048 *k, char *e)
{
	char line[STRLEN], *homedir, *username;
	FILE *f1;

	/* Set up defaults */
	bzero(k, sizeof(struct k8048));
	strncpy(k->device, SERIAL_DEVICE, STRLEN);
	k->bitrules = (PGD_IN_PULLUP | PGD_OUT_FLIP | PGC_OUT_FLIP | VPP_OUT_FLIP | PGD_IN_FLIP);
	k->busy = BUSY;
	k->fd = -1;
        k->gpio.fd = -1;
	k->gpio.tx = GPIO_TX;
	k->gpio.rts = GPIO_RTS;
	k->gpio.dtr = GPIO_DTR;
	k->gpio.cts = GPIO_CTS; 
	k->sleep = 1;
	k->fwsleep = 1;

	/* Default to current working directory */
	snprintf(k->dotfile, STRLEN, "%s/%s", getcwd(line, STRLEN), DOTFILENAME);
	if (access(k->dotfile, R_OK) < 0) {
		/* Try home dir */
		homedir = getenv("HOME");
		if (homedir != NULL) {
			snprintf(k->dotfile, STRLEN, "%s/%s", homedir, DOTFILENAME);
		} else {
			/* Try default home dir */
			username = getenv("USER");
			if (username != NULL) {
				snprintf(k->dotfile, STRLEN, "/home/%s/%s", username, DOTFILENAME);
			}
		}
	}
	k->dotfile[STRMAX] = '\0';
#ifdef DEBUG
	printf("DOTFILE=%s\n", k->dotfile);
#endif
	/* Load dot file if present */
	f1 = fopen(k->dotfile, "rb");
	if (f1 != NULL) {
		while (fgets(line, STRLEN, f1) != NULL) {
			line[strlen(line) - 1] = '\0'; /* Remove LF */

			if (mystrcasestr(line, "DEVICE=") == line) {
#ifdef DEBUG
				printf("DEVICE=%s\n", &line[7]);
#endif
				strncpy(k->device, &line[7], STRLEN);
			}
			else if (mystrcasestr(line, "SLEEP=") == line) {
#ifdef DEBUG
				printf("SLEEP=%s\n", &line[6]);
#endif
				k->sleep = strtoul(&line[6], NULL, 0);
			}
			else if (mystrcasestr(line, "BITRULES=") == line) {
#ifdef DEBUG
				printf("BITRULES=%s\n", &line[9]);
#endif
				k->bitrules = strtoul(&line[9], NULL, 0);
			}
			else if (mystrcasestr(line, "BUSY=") == line) {
#ifdef DEBUG
				printf("BUSY=%s\n", &line[5]);
#endif
				k->busy = strtoul(&line[5], NULL, 0);
			}
			else if (mystrcasestr(line, "TX=") == line) {
#ifdef DEBUG
				printf("TX=%s\n", &line[3]);
#endif
				k->gpio.tx = strtoul(&line[3], NULL, 0);
			}
			else if (mystrcasestr(line, "RTS=") == line) {
#ifdef DEBUG
				printf("RTS=%s\n", &line[4]);
#endif
				k->gpio.rts = strtoul(&line[4], NULL, 0);
			}
			else if (mystrcasestr(line, "DTR=") == line) {
#ifdef DEBUG
				printf("DTR=%s\n", &line[4]);
#endif
				k->gpio.dtr = strtoul(&line[4], NULL, 0);
			}
			else if (mystrcasestr(line, "CTS=") == line) {
#ifdef DEBUG
				printf("CTS=%s\n", &line[4]);
#endif
				k->gpio.cts = strtoul(&line[4], NULL, 0);
			}
			else if (mystrcasestr(line, "FWSLEEP=") == line) {
#ifdef DEBUG
				printf("FWSLEEP=%s\n", &line[8]);
#endif
				k->fwsleep = strtoul(&line[8], NULL, 0);
			}
			else if (mystrcasestr(line, "DEBUG=") == line) {
#ifdef DEBUG
				printf("DEBUG=%s\n", &line[6]);
#endif
				k->debug = strtoul(&line[6], NULL, 0);
			}
		}
		fclose(f1);
	} else { /* We are using the defaults */
		snprintf(k->dotfile, STRLEN, "Using defaults (override in %s)", DOTFILENAME);
	}
}
